package cp;

import cp.db.DB;
import cp.db.Settings;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

public class Players extends Vector<Player> {
  private Settings settings;

  public Players(Settings settings) {
    super();
    this.settings = settings;
  }
  
  public void load() throws SQLException {
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    String sql = "SELECT * FROM availableplayers " + 
                 "ORDER BY lastname, firstname, ratingn ";
    ResultSet rs = s.executeQuery(sql);
    while (rs.next()) {
      Player player = new Player(settings);
      player.load(rs);
      add(player);
    }
    s.close();
  }
  
  void load(Tournament tournament) throws SQLException {
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    String sql = "SELECT * FROM tournamentplayers " + 
                 "WHERE tournamentindex = " + tournament.getIndex() + 
                 "ORDER BY lastname, firstname, ratingn ";
    ResultSet rs = s.executeQuery(sql);
    while (rs.next()) {
      Player player = new Player(settings);
      player.load(rs);
      player.loadFromTournament(rs);
      add(player);
    }
    s.close();
  }
  
  Player findPlayer(int index) {
    for (int i=0; i<size(); i++) {
      Player player = get(i);
      if (player.getIndex() == index) {
        return player;
      }
    }
    return null;
  }
  
  void add(Player player, Tournament tournament) throws SQLException {
    player.add(tournament);
    add(player);
  }

  void remove(Player player, Tournament tournament) throws SQLException {
    player.remove(tournament);
    remove(player);
  }

  void update(Player player, Tournament tournament) throws SQLException {
    player.update(tournament);
  }

  public int getNextLotNr() {
    int highestLotNr = 0;
    for (int i=0; i<size(); i++) {
      Player player = get(i);
      if (player.getLotNr() > highestLotNr) {
        highestLotNr = player.getLotNr(); 
      }
    }
    return highestLotNr+1;
  }

  public void setLotNr(Round round) {
    if (round != null) {
      for (int i = 0; i < size(); i++) {
        Player player = get(i);
        try {
          player.setLotNr(round.getLotNr(player));
        }
        catch (ChessPairerException e) {
          player.setLotNr(Integer.MAX_VALUE);
        }
      }
    }
  }
  
  public Player getPlayer(String firstName, String lastName, Club club) {
    for (int i=0; i<size(); i++) {
      Player player = get(i);
      if (player.getFirstName().equalsIgnoreCase(firstName) && player.getLastName().equalsIgnoreCase(lastName) && player.getClub() == club) {
        return player;
      }
    }
    return null;
  }
  
  public boolean isOneClubInMajority() {
    // Fill Hashtable with members
    Hashtable<Club,Integer> table = new Hashtable<Club,Integer>();
    for (int i=0; i<size(); i++) {
      Club club = get(i).getClub();
      if (club != null) {
        int members = 0;
        if (table.containsKey(club)) {
          // Get members
          members = table.get(club);
        }
        table.put(club, new Integer(members+1));
      }
    }
    // Get the highest member number
    Collection<Integer> coll = table.values();
    Iterator<Integer> it = coll.iterator();
    int max = 0;
    while (it.hasNext()) {
      int value = it.next();
      if (value > max) {
        max = value;
      }
    }
    return (max > this.size()/2);
  }

  public int playsIn(Club club) {
    int ans = 0;
    for (int i=0; i<size(); i++) {
      if (get(i).getClub() == club) {
        ans++;
      }
    }
    return ans;
  }

  Clubs getClubs() {
    Clubs clubs = new Clubs();
    for (int i=0; i<size(); i++) {
      Club club = get(i).getClub();
      if (club != null && !clubs.contains(club)) {
        clubs.add(club);
      }
    }
    return clubs;
  }
}
