package cp;

import cp.pairing.Berger;
import cp.pairing.Monrad;
import cp.pairing.NordicSchweizer;
import cp.pairing.PairingSystem;

import cp.tiebreak.Tiebreak;
import cp.tiebreak.Tiebreaks;

import java.io.FileWriter;
import java.sql.Connection;
import cp.db.DB;
import cp.db.Settings;

import java.io.IOException;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import java.util.Vector;

import javax.swing.JOptionPane;

public class Tournament {
  private String name;
  private String group;
  private String pairingSystem = "Monrad";
  private String initialPairing = "Slumpad";
  private int nrOfRounds = 0;
  private boolean barredPairing = false;
  private boolean compensateWeakPlayerPP = false;
  private int pointsPerGame = 1;
  private boolean isChess4 = false;
  private int index = -1;
  private Players players;
  private Rounds rounds = new Rounds(this);
  private String resultsPage;
  private String standingsPage;
  private String playerListPage;
  private String roundForRoundPage;
  private String clubStandingsPage;
  private Settings settings;
  private Tiebreaks tiebreaks = new Tiebreaks();

  public Tournament(Settings settings) {
    this.settings = settings;
    players = new Players(settings);
  }
  
  public boolean load(String tournament, String group) throws SQLException, ChessPairerException {
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    String sql = "SELECT * FROM tournaments " + 
                 "WHERE tournament = '" + tournament + "' AND tournamentgroup = '" + group + "'";
    ResultSet rs = s.executeQuery(sql);
    if (rs.next()) {
      return load(rs.getInt("index"));
    }
    return false;
  }

  public boolean load(int index) throws SQLException, ChessPairerException {
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    String sql = "SELECT * FROM tournaments " + 
                 "WHERE index = " + index;
    ResultSet rs = s.executeQuery(sql);
    if (rs.next()) {
      load(rs);
      // Load tournament player
      loadPlayers();
      // Load rounds
      loadRounds();
      // Load tiebreak systems
      loadTiebreaks();
      return true;
    }
    return false;
  }
  
  private void load(ResultSet rs) throws SQLException {
    name = rs.getString("tournament");
    group = rs.getString("tournamentgroup");
    pairingSystem = rs.getString("pairingsystem");
    initialPairing = rs.getString("initialpairing");
    nrOfRounds = rs.getInt("rounds");
    barredPairing = rs.getBoolean("barredpairing");
    compensateWeakPlayerPP = rs.getBoolean("compensateweakplayerpp");
    pointsPerGame = rs.getInt("pointspergame");
    resultsPage = rs.getString("resultspage");
    standingsPage = rs.getString("standingspage");
    playerListPage = rs.getString("playerlistpage");
    roundForRoundPage = rs.getString("roundforroundpage");
    clubStandingsPage = rs.getString("clubStandingsPage");
    index = rs.getInt("index");
    isChess4 = rs.getBoolean("chess4");
  }

  private void loadPlayers() throws SQLException {
    // Load tournament player
    players = new Players(settings);
    players.load(this);
  }

  private void loadTiebreaks() throws SQLException {
    // Load tournament tiebreaks
    tiebreaks = new Tiebreaks();
    tiebreaks.load(this);
  }
  
  public void save() throws SQLException {
    Connection conn = DB.getConnection();
    String sql = "INSERT INTO tournaments " + 
                 "(tournament, tournamentgroup, pairingsystem, initialpairing, rounds, barredpairing, compensateweakplayerpp, pointspergame, resultspage, standingspage, playerlistpage, roundforroundpage, chess4, clubstandingspage) " + 
                 "VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
    PreparedStatement s = conn.prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
    s.setString(1, name);
    s.setString(2, group);
    s.setString(3, pairingSystem);
    s.setString(4, initialPairing);
    s.setInt(5, nrOfRounds);
    s.setBoolean(6, barredPairing);
    s.setBoolean(7, compensateWeakPlayerPP);
    s.setInt(8, pointsPerGame);
    s.setString(9, resultsPage);
    s.setString(10, standingsPage);
    s.setString(11, playerListPage);
    s.setString(12, roundForRoundPage);
    s.setBoolean(13, isChess4);
    s.setString(14, clubStandingsPage);
    try {
      int rows = s.executeUpdate();
      if (rows != 1) {
        throw new SQLException(rows + " rader ndrades, frvntade 1 rad");
      }
      // Read the generated index
      ResultSet rs = s.getGeneratedKeys();
      rs.next();
      index = rs.getInt(1);
      saveTiebreak();
      conn.commit();
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
    s.close();
  }

  public void saveTiebreak() throws SQLException {
    Connection conn = DB.getConnection();
    Vector<Tiebreak> selectedTiebreaks = tiebreaks.getSelectedTiebreaks();
    for (int i=0; i<selectedTiebreaks.size(); i++) {
      String sql = "INSERT INTO tournamenttiebreaks " + 
                   "(tiebreak, tournamentindex) " + 
                   "VALUES (?,?)";
      PreparedStatement s = conn.prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
      s.setString(1, selectedTiebreaks.get(i).toString());
      s.setInt(2, index);
      int rows = s.executeUpdate();
      if (rows != 1) {
        throw new SQLException(rows + " rader ndrades, frvntade 1 rad");
      }
      s.close();
    }
  }

  public void update() throws SQLException {
    Connection conn = DB.getConnection();
    String sql = "UPDATE tournaments " + 
                 "SET pairingsystem=?, initialpairing=?, rounds=?, barredpairing=?, compensateweakplayerpp=?, pointspergame=?, resultspage=?, standingspage=?, playerlistpage=?, roundforroundpage=?, chess4=?, clubstandingspage=?" +
                 "WHERE index=" + index;
    PreparedStatement s = conn.prepareStatement(sql);
    s.setString(1, pairingSystem);
    s.setString(2, initialPairing);
    s.setInt(3, nrOfRounds);
    s.setBoolean(4, barredPairing);
    s.setBoolean(5, compensateWeakPlayerPP);
    s.setInt(6, pointsPerGame);
    s.setString(7, resultsPage);
    s.setString(8, standingsPage);
    s.setString(9, playerListPage);
    s.setString(10, roundForRoundPage);
    s.setBoolean(11, isChess4);
    s.setString(12, clubStandingsPage);
    try {
      int rows = s.executeUpdate();
      if (rows != 1) {
        throw new SQLException(rows + " rader ndrades, frvntade 1 rad");
      }
      updateTiebreak();
      conn.commit();
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
    s.close();
  }

  public void setName(String name) {
    this.name = name;
  }

  public void setGroup(String group) {
    this.group = group;
  }

  public String getGroup() {
    return group;
  }

  public void setPairingSystem(String pairingSystem) {
    this.pairingSystem = pairingSystem;
  }

  public String getPairingSystem() {
    return pairingSystem;
  }

  public void setInitialPairing(String initialPairing) {
    this.initialPairing = initialPairing;
  }

  public String getInitialPairing() {
    return initialPairing;
  }

  public void setNrOfRounds(int rounds) {
    this.nrOfRounds = rounds;
  }

  public int getNrOfRounds() {
    return nrOfRounds;
  }

  public void setBarredPairing(boolean barredPairing) {
    this.barredPairing = barredPairing;
  }

  public boolean isBarredPairing() {
    return barredPairing;
  }
    
  public void setCompensateWeakPlayerPP(boolean compensateWeakPlayerPP) {
    this.compensateWeakPlayerPP = compensateWeakPlayerPP;
  }

  public boolean isCompensateWeakPlayerPP() {
    return compensateWeakPlayerPP;
  }

  public void setIsChess4(boolean isChess4) {
    this.isChess4 = isChess4;
  }

  public boolean isChess4() {
    return isChess4;
  }

  public void setPointsPerGame(int pointsPerGame) {
    this.pointsPerGame = pointsPerGame;
  }

  public int getPointsPerGame() {
    return pointsPerGame;
  }

  public int getIndex() {
    return index;
  }

  public String getName() {
    return name;
  }
  
  public String toString() {
    return name + " " + group;
  }
  
  public void delete() throws SQLException {
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    try {
      // Delete tournament games
      String sql = "DELETE from tournamentgames " +
                   "WHERE tournament = " + index;
      s.execute(sql);
      // Delete tournament players
      sql = "DELETE from tournamentplayers " +
                   "WHERE tournamentindex = " + index;
      s.execute(sql);
      // Delete tournament tiebreaks
      sql = "DELETE from tournamenttiebreaks " +
                   "WHERE tournamentindex = " + index;
      s.execute(sql);
      // Delete tournament
      sql = "DELETE from tournaments " +
                   "WHERE index = " + index;
      s.execute(sql);
      conn.commit();
    }
    catch (SQLException e) {
      conn.rollback();
      throw e;
    }
    s.close();
  }
  
  public void add(Player player) throws SQLException {
    players.add(player, this);
  }
  
  public void remove(Player player) throws SQLException {
    players.remove(player, this);
 }
  
  public int nrOfPlayers() {
    return players.size();
  }
  
  public Rounds getRounds() {
    return rounds;
  }
  
  public Round getCurrentRound() {
    if (rounds.size() == 0) {
      return null;
    }
    else if (pairingSystem.equals("Berger")) {
      // Figure out the last round
      return rounds.getCurrentBergerRound();
    }
    return rounds.get(rounds.size()-1);
  }
  
  public boolean isFinished() {
    return rounds.size() == nrOfRounds;
  }
  
  public void pair() throws ChessPairerException, SQLException, ChessPairerInfoException {
    if (players.size() < 2) {
      throw new ChessPairerInfoException("Lgg till ngra spelare frst!");
    }
    if ( rounds.size() >= nrOfRounds) {
      throw new ChessPairerInfoException("Alla ronder r spelade! Vill du lgga till fler, ndra turneringsinstllningarna.");
    }
    // Check that all games have results!
    if (rounds.size() > 0) {
      Rounds rounds = getRounds();
      for (int i=0; i<rounds.size(); i++) {
        Round round = rounds.get(i);
        if (!round.hasAllResults()) {
          throw new ChessPairerInfoException("Alla matcher i rond " + round.getRoundNr() + " mste ha resultat innan nsta rond kan lottas!");
        }
      }
    }
    PairingSystem pairing = null;
    if (pairingSystem.equals("Monrad")) {
      pairing = new Monrad(this);
    }
    else if (pairingSystem.equals("Nordisk Schweizer") ||pairingSystem.equals("Nordic Schweizer") ) {
      pairing = new NordicSchweizer(this);
    }
    else if (pairingSystem.equals("Berger")) {
      if (rounds.size() > 0) {
        throw new ChessPairerInfoException("Det finns redan lottade ronder! Berger lottar alla ronder i ett svep!");
      }
      if (this.getNrOfRounds() < players.size()-1) {
        throw new ChessPairerInfoException(String.format("Fr att lotta en Berger-turnering med %d spelare krvs minst %d ronder.", players.size(), players.size()-1));
      }
      if (this.getNrOfRounds() > players.size()) {
        if (JOptionPane.showConfirmDialog(null, String.format("Endast %d ronder behvs. Fler kommer att ge frironder. Vill du lotta nd?", players.size()-1), "Bekrfta lottning", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) != JOptionPane.OK_OPTION) {
          return;
        }
      }
      pairing = new Berger(this);
      // Pair all rounds
      for (int i=0; i<getNrOfRounds(); i++) {
        Games games = pairing.pair();
        // Add games to players
        games.addToPlayers();
        // Add round
        Round round = new Round(this, i+1, games);
        rounds.add(round);
        // Save pairing
        round.save();
      }
      return;
    }
    else {
      throw new ChessPairerInfoException("Lottningssystem " + pairingSystem + " ej implementerat.");
    }
    // Pair next round
    Games games = pairing.pair();
    if (games == null || games.size() == 0) {
      if (this.isBarredPairing()) {
        throw new ChessPairerInfoException("Kan inte lotta. Lottning r inte mjlig!\nFr att lsa detta, ta bort laglottning. Gr annars en manuell lottning eller avsluta turneringen.");
      }
      else {
        throw new ChessPairerInfoException("Kan inte lotta. Lottning r inte mjlig!\nFr att lsa detta, gr en manuell lottning eller avsluta turneringen.");
      }
    }
    else {
      // Add games to players
      games.addToPlayers();
      // Add round
      Round round = new Round(this, rounds.size()+1, games);
      rounds.add(round);
      // Save pairing
      round.save();
    }
  }

  public void unpair() throws SQLException {
    if (pairingSystem.equals("Berger")) {
      if (JOptionPane.showConfirmDialog(null, String.format("Alla ronder kommer att tas bort fr en Berger-turnering. r du sker p att du vill ta bort alla ronder?", players.size()-1), "Bekrfta ngra lottning", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == JOptionPane.OK_OPTION) {
        for (int i=rounds.size(); i>0; i--) {
          unpairLastRound();
        }
      }
    }
    else {
      unpairLastRound();
    }
  }
    
  private void unpairLastRound() throws SQLException {
    // Remove the round games from all player
    Round round = rounds.get(rounds.size()-1);
    Games games = round.getGames();
    games.removeFromPlayers();
    // Remove the round
    rounds.delete(rounds.size());
    // Update player lot nr to be what it was before the pairing. This is saved in the round
    if (rounds.size() > 0) {
      players.setLotNr(rounds.get(rounds.size()-1));
    }  
  }
  
  boolean isInTournament(Player player) {
    return players.contains(player);
  }

  public void update(Player player) throws SQLException {
    players.update(player, this);
  }

  public Players getPlayers() {
    return players;
  }

  private void loadRounds() throws SQLException, ChessPairerException {
    rounds = new Rounds(this);
    rounds.load(nrOfRounds, players);
  }

  public void setResultsPage(String resultsPage) {
    this.resultsPage = resultsPage;
  }

  public String getResultsPage() {
    return resultsPage;
  }

  public void setStandingsPage(String standingsPage) {
    this.standingsPage = standingsPage;
  }

  public String getStandingsPage() {
    return standingsPage;
  }

  public void setPlayerListPage(String playerListPage) {
    this.playerListPage = playerListPage;
  }

  public String getPlayerListPage() {
    return playerListPage;
  }

  public void setRoundForRoundPage(String roundForRoundPage) {
    this.roundForRoundPage = roundForRoundPage;
  }

  public String getRoundForRoundPage() {
    return roundForRoundPage;
  }

  public void setClubStandingsPage(String clubStandingsPage) {
    this.clubStandingsPage = clubStandingsPage;
  }

  public String getClubStandingsPage() {
    return clubStandingsPage;
  }

  public Settings getSettings() {
    return settings;
  }

/*  public int getRoundNr(Game game) {
    for (int i = 0; i < rounds.size(); i++) {
      Round round = rounds.get(i);
      Games games = round.getGames();
      if (games.contains(game)) {
        return round.getRoundNr();
      }
    }
    return Integer.MAX_VALUE;
  }*/

  public Tiebreaks getTiebreaks() {
    return tiebreaks;
  }
  
  public void setTiebreaks(Tiebreaks tiebreaks) {
    this.tiebreaks = tiebreaks;
  }

  public Vector<Tiebreak> getSelectedTiebreaks() {
    return tiebreaks.getSelectedTiebreaks();
  }

  private void updateTiebreak() throws SQLException {
    // Delete old tiebreak
    Connection conn = DB.getConnection();
    Statement s = conn.createStatement();
    // Delete tournament tiebreaks
    String sql = "DELETE from tournamenttiebreaks " +
                   "WHERE tournamentindex = " + index;
    s.execute(sql);
    s.close();
    // Save new tiebreak
    saveTiebreak();
  }

  public Vector<Club> getCurrentClubList() {
    // First get all clubs and remove players from them.
    Clubs clubs = players.getClubs();
    clubs.clearPlayers();
    // Add players to the clubs
    for (int i=0; i<players.size(); i++) {
      Player player = players.get(i);
      Club club = player.getClub();
      if (club != null) {
        club.add(player);
      }
    }
    return clubs;
  }

}
