package cp.gui;

import cp.ChessPairerException;
import cp.Round;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.print.PrinterException;
import java.text.MessageFormat;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ScrollPaneConstants;
import javax.swing.table.TableColumn;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

public class AlphabeticalPairingPanel extends JScrollPane {
  private AlphabeticalPairingTableModel tableModel = new AlphabeticalPairingTableModel();
  private JTable jTable = new JTable(tableModel);

  public AlphabeticalPairingPanel() {
    try {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    this.setViewportView(jTable);
    jTable.setFillsViewportHeight(true);
    setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
    setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
    this.addComponentListener(new ComponentAdapter() {
        public void componentShown(ComponentEvent e) {
          this_componentShown();
        }
      });
    jTable.setShowGrid(false);
    jTable.setShowHorizontalLines(true);
  }
  
  public void setRound(Round round) {
		tableModel.setRound(round);
		setColumnWidths();
	}

  public void print() throws PrinterException {
    update();
		Round round = tableModel.getRound();
		if (round != null) {
			MessageFormat headerFormat = new MessageFormat(String.format("Alfabetisk lottning fr rond %d", round.getRoundNr()));
			MessageFormat footerFormat = new MessageFormat(String.format("%s, grupp %s\n     - Sida {0} -", round.getTournament(), round.getTournament().getGroup()));
			jTable.print(JTable.PrintMode.FIT_WIDTH, headerFormat, footerFormat);
		}
	}

	public Round getRound() {
    return tableModel.getRound();
  }

  public void update() {
		if (tableModel != null) {
      tableModel.update();
			setColumnWidths();
		  jTable.doLayout();
      jTable.repaint();
		}
	}
  
  private void setColumnWidths() {
    TableColumn column = null;
    for (int i = 0; i < jTable.getColumnModel().getColumnCount(); i++) {
      column = jTable.getColumnModel().getColumn(i);
      if (i == 0) {
        column.setPreferredWidth(200); 
      }
      else if (i == 1) {
        column.setPreferredWidth(150);
      }
      else if (i == 2) {
        column.setPreferredWidth(40);
      }
      else if (i == 3) {
        column.setPreferredWidth(40);
      }
      else if (i == 4) {
        column.setPreferredWidth(40);
      }
    }
  }
  
	private void insertHeaders(Document document, Element table, int columns) {
		// Create headers
		Element tr = document.createElement("tr");
		table.appendChild(tr);
		tr.setAttribute("class", "CP_TableAlphabeticalHeader");
		for (int cols=0; cols<columns; cols++) {
			for (int i = 0; i < tableModel.getColumnCount(); i++) {
				Element th = document.createElement("th");
				tr.appendChild(th);
				th.setAttribute("class", String.format("CP_TableAlphabeticalHeaderC%d", i+1));
				Text text = document.createTextNode(tableModel.getColumnName(i));
				th.appendChild(text);
			}
		}
	}
	
	public void save(Document document, Element body, int maxRows, int columns) throws ChessPairerException {
	  // Insert table
	  Element table = document.createElement("table");
	  body.appendChild(table);
	  table.setAttribute("class", "CP_TableAlphabetical");
	  table.setAttribute("cellpadding", "0");
	  table.setAttribute("cellspacing", "0");
	  table.setAttribute("border", "0");
		
		int modelOffset=0;
		// Calculate nr of rows needed
		int neededRows = (int)Math.ceil(tableModel.getRowCount()/(double)columns);
		int rows = neededRows;
		if (neededRows > maxRows) {
			rows = Math.min(neededRows, maxRows);
		}
		// Create rows
		for (int repeat=0; repeat < Math.ceil(tableModel.getRowCount()/((double)rows*columns)); repeat++) {
			insertHeaders(document, table, columns);
		  for (int i = 0; i < rows; i++) {
				createPlayersRow(document, table, columns, rows, i, modelOffset);
			}
		  modelOffset += columns*rows;
		}
	}

	private void createPlayersRow(Document document, Element table, int columns, int rows, int row, int modelOffset) {
		Element tr = document.createElement("tr");
		table.appendChild(tr);
		tr.setAttribute("class", "CP_TableAlphabeticalRow");
		// Create columns
		for (int column=0; column<columns; column++) {
			// Create subcolumns in each row
			createPlayersColumn(document, tr, column, row, column*rows + modelOffset);
		}
	}

	private void createPlayersColumn(Document document, Element tr, int column, int row, int modelOffset) {
		for (int j = 0; j < tableModel.getColumnCount(); j++) {
			Element th = document.createElement("th");
			tr.appendChild(th);
			boolean even = column % 2 == 1;
			th.setAttribute("class", String.format("CP_TableAlphabeticalRowC%d%s", j+1, even ? "E" : "O"));  
			int index = row + modelOffset;
			if (tableModel.getRowCount() > index) {
				Object obj = tableModel.getValueAt(index, j);
				if (obj != null) {
					Text text = document.createTextNode(obj.toString());
					th.appendChild(text);
				}
			}
		}
	}

  /**
   * When the table is shown, set the round again so the player information is correct.
   * @param e
   */
  private void this_componentShown() {
    update();
  }
}
