package cp.gui;

import cp.Club;
import cp.Tournament;
import java.sql.SQLException;
import java.util.Collections;
import java.util.Vector;
import javax.swing.JOptionPane;
import javax.swing.table.AbstractTableModel;

public class Chess4SetupTableModel extends AbstractTableModel {
  private String[] columnNames = {"Klass", "Spelare", "Klasstorlek"};
  private Chess4SetupComparator comparator = new Chess4SetupComparator();
  private Vector<Club> currentClubs = new Vector<Club>();
	private Tournament tournament;

  public String getColumnName(int col) {
    return columnNames[col].toString();
  }

  public int getRowCount() {
    if (tournament != null) {
      return currentClubs.size();
    }
    else {
      return 2;
    }
  }

  public int getColumnCount() {
    return columnNames.length;
  }

  public Object getValueAt(int rowIndex, int columnIndex) {
    if (tournament != null) {
			Club club = currentClubs.get(rowIndex);
      if (columnIndex == 0) {
        return club.toString();
      }
      else if (columnIndex == 1) {
        return tournament.getPlayers().playsIn(club);
      }
      else if (columnIndex == 2) {
        return club.getChess4Members();
      }
    }
    return null;
  }
  
  public void setValueAt(Object value, int rowIndex, int columnIndex) {
		Club club = currentClubs.get(rowIndex);
    String tmp = (String) value;
    int oldValue = club.getChess4Members();
    try {
      int members = Integer.parseInt(tmp);
      club.setChess4Members(members);
      club.update();
    }
    catch (NumberFormatException e) {
      JOptionPane.showMessageDialog(null, "Kan inte omvandla till ett tal: " + e.getMessage(), "Information", JOptionPane.INFORMATION_MESSAGE);
      club.setChess4Members(oldValue);
    }
    catch (SQLException e) {
      JOptionPane.showMessageDialog(null, "Kan inte spara klasstorlek i databasen! Felrkning kan uppkomma vid varje ny rond!! Frsk att starta om programmet. SQL fel:" + e.getMessage(), "Fel", JOptionPane.ERROR_MESSAGE);
      club.setChess4Members(oldValue);
    }
  }

  public boolean isCellEditable(int row, int col) {
    if (col == 2) {
      return true;
    }
    return false;
  }

  public void update() {
		if (tournament != null) {
			currentClubs = tournament.getCurrentClubList();
		  Collections.sort(currentClubs, comparator);
		}
		else {
			currentClubs = new Vector<Club>();
		}
    fireTableDataChanged();
  }

	void setTournament(Tournament tournament) {
		this.tournament = tournament;
		update();
	}
}
