package cp.gui;

import cp.Club;
import cp.Round;

import java.awt.BorderLayout;

import java.awt.print.PrinterException;

import java.sql.SQLException;

import java.text.MessageFormat;

import java.util.Vector;

import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ScrollPaneConstants;
import javax.swing.table.TableColumn;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

public class ClubStandingsPanel extends JScrollPane {
  private ClubStandingsTableModel standingsTableModel = new ClubStandingsTableModel();
  private JTable jTable = new JTable(standingsTableModel);

  public ClubStandingsPanel() {
    try {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    this.setViewportView(jTable);
    jTable.setFillsViewportHeight(true);
    setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
    setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
    jTable.setShowGrid(false);
    jTable.setShowHorizontalLines(true);
  }
  
  public void setRound(Round round, Vector<Club> currentClubs) {
		standingsTableModel.setRound(round, currentClubs);
		update();
	}

  public void print() throws PrinterException {
		Round round = standingsTableModel.getRound();
		if (round != null) {
			MessageFormat headerFormat = new MessageFormat(String.format("Klubbstllning efter rond %d", round.getRoundNr()));
			MessageFormat footerFormat = new MessageFormat(String.format("%s, grupp %s\n     - Sida {0} -", round.getTournament(), round.getTournament().getGroup()));
			jTable.print(JTable.PrintMode.FIT_WIDTH, headerFormat, footerFormat);
		}
	}

	public Round getRound() {
		return standingsTableModel.getRound();
	}

  public void save(Document document, Element body) {
		// Insert table
		Element table = document.createElement("table");
		body.appendChild(table);
		table.setAttribute("class", "CP_TableStandings");
		table.setAttribute("cellpadding", "0");
		table.setAttribute("cellspacing", "0");
		table.setAttribute("border", "0");
		// Create headers
		Element tr = document.createElement("tr");
		tr.setAttribute("class", "CP_TableStandingsHeader");
		table.appendChild(tr);
		for (int i = 0; i < standingsTableModel.getColumnCount(); i++) {
			Element th = document.createElement("th");
			tr.appendChild(th);
			th.setAttribute("class", String.format("CP_TableClubStandingsHeaderC%d", i + 1));
			Text text = document.createTextNode(standingsTableModel.getColumnName(i));
			th.appendChild(text);
		}
		for (int i = 0; i < standingsTableModel.getRowCount(); i++) {
			tr = document.createElement("tr");
			table.appendChild(tr);
			tr.setAttribute("class", "CP_TableStandingsRow");
			for (int j = 0; j < standingsTableModel.getColumnCount(); j++) {
				Element th = document.createElement("th");
				tr.appendChild(th);
				th.setAttribute("class", String.format("CP_TableClubStandingsC%d", j + 1));
				Object obj = standingsTableModel.getValueAt(i, j);
				if (obj != null) {
					Text text = document.createTextNode(obj.toString());
					th.appendChild(text);
				}
			}
		}
	}

  public void update() {
		if (standingsTableModel != null) {
			standingsTableModel.update();
			setColumnWidths();
		}
	  jTable.doLayout();
	  jTable.repaint();
	}
  
  private void setColumnWidths() {
    TableColumn column = null;
    for (int i = 0; i < jTable.getColumnModel().getColumnCount(); i++) {
      column = jTable.getColumnModel().getColumn(i);
      if (i == 0) {
        column.setPreferredWidth(30); 
      }
      else if (i == 1) {
        column.setPreferredWidth(200);
      }
      else if (i == 2) {
        column.setPreferredWidth(45);
      }
    }
  }
  
}
