package cp.gui;

import cp.Tournament;

import cp.db.*;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.print.PrinterException;
import java.text.MessageFormat;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ScrollPaneConstants;
import javax.swing.table.*;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

public class PlayerListPanel extends JScrollPane {
  private TournamentPlayerListTableModel playerListTableModel;
  private JTable jTable;

  public PlayerListPanel(Settings settings) {
    try {
      playerListTableModel = new TournamentPlayerListTableModel(settings);
      jTable = new JTable(playerListTableModel);
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    this.setViewportView(jTable);
    jTable.setFillsViewportHeight(true);
    setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
    setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
    this.addComponentListener(new ComponentAdapter() {
        public void componentShown(ComponentEvent e) {
          this_componentShown();
        }
      });
    jTable.setShowGrid(false);
    jTable.setShowHorizontalLines(true);
  }
  
  void setTournament(Tournament tournament) {
    playerListTableModel.setTournament(tournament);
  }

  void print() throws PrinterException {
		Tournament tournament = playerListTableModel.getTournament();
    if (tournament != null) {
      MessageFormat headerFormat = new MessageFormat(String.format("Spelare i turnering %s, grupp %s", tournament, tournament.getGroup()));
      jTable.print(JTable.PrintMode.FIT_WIDTH, headerFormat, null);
    }
  }

  public void save(Document document, Element body) {
		// Insert table
		Element table = document.createElement("table");
		body.appendChild(table);
		table.setAttribute("class", "CP_TablePlayers");
		table.setAttribute("cellpadding", "0");
		table.setAttribute("cellspacing", "0");
		table.setAttribute("border", "0");
		// Create headers
		Element tr = document.createElement("tr");
		tr.setAttribute("class", "CP_TablePlayersHeader");
		table.appendChild(tr);
		for (int i = 0; i < playerListTableModel.getColumnCount(); i++) {
			Element th = document.createElement("th");
			tr.appendChild(th);
			th.setAttribute("class", String.format("CP_TablePlayersHeaderC%d", i + 1));
			Text text = document.createTextNode(playerListTableModel.getColumnName(i));
			th.appendChild(text);
		}
		for (int i = 0; i < playerListTableModel.getRowCount(); i++) {
			tr = document.createElement("tr");
			table.appendChild(tr);
			tr.setAttribute("class", "CP_TablePlayersRow");
			for (int j = 0; j < playerListTableModel.getColumnCount(); j++) {
				Element th = document.createElement("th");
				tr.appendChild(th);
				th.setAttribute("class", String.format("CP_TablePlayersC%d", j + 1));
				Object obj = playerListTableModel.getValueAt(i, j);
				if (obj != null) {
					Text text = document.createTextNode(obj.toString());
					th.appendChild(text);
				}
			}
		}
	}

	public Tournament getTournament() {
		return playerListTableModel.getTournament();
	}
  
  public void update() {
    if (playerListTableModel != null) {
      playerListTableModel.update();
      playerListTableModel.setColumnWidths(jTable);
    }
    jTable.doLayout();
    jTable.repaint();
  }

  private void this_componentShown() {
    update();
  }
}
