package cp.gui;

import cp.Player;
import cp.Players;
import cp.Result;
import cp.Round;
import cp.db.*;
import cp.tiebreak.Tiebreak;
import java.util.Collections;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;

public class StandingsTableModel extends AbstractTableModel {
  Vector<String> columnNames = new Vector<String>();
  private Round round;
  private StandingsComparator standingsComparator;
  private Players players;
  private Settings settings;

  StandingsTableModel(Settings settings) {
    this.settings = settings;
    initColumnNames();
  }

  public void setRound(Round round) {
    this.round = round;
    update();
  }
  
  public String getColumnName(int col) {
    return columnNames.get(col);
  }

  public int getRowCount() {
    if (players != null) {
      return players.size();
    }
    else {
      return 2;
    }
  }

  public int getColumnCount() {
    return columnNames.size();
  }

  public Object getValueAt(int rowIndex, int columnIndex) {
    if (round != null) {
			Player player = players.get(rowIndex);
      Vector<Tiebreak> tiebreaks = round.getTournament().getSelectedTiebreaks();
      if (columnIndex == 0) {
        if (rowIndex > 0) {
					Player previousPlayer = players.get(rowIndex-1);
          if (standingsComparator.compare(player, previousPlayer) == 0) {
            return null;
          }
        }
        return rowIndex+1;
      }
      else if (columnIndex == 1) {
        return player.toString();
      }
      else if (columnIndex == 2) {
        return player.getGroup();
      }
      else if (columnIndex == 3) {
        return player.getClub();
      }
      else if (getColumnName(columnIndex).equalsIgnoreCase("Rating")) {
        return (player.getRatingN() > 0 ? player.getRatingN() : "");
      }
      else if (getColumnName(columnIndex).equalsIgnoreCase("ELO")) {
        return (player.getRatingI() > 0 ? player.getRatingI() : "");
      }
      else if (getColumnName(columnIndex).equalsIgnoreCase("Pong")) {
        return Result.toString(player.getScore(round));
      }
      // Check tiebrek columns
      for (int i=0; i<tiebreaks.size(); i++) {
        Tiebreak tiebreak = tiebreaks.get(i);
        if (getColumnName(columnIndex).equalsIgnoreCase(tiebreak.getShortName())) {
          return player.getTiebreak(tiebreak, round);
        }
      }
    }
    return null;
  }
  
  public boolean isCellEditable(int row, int col) {
    return false;
  }

	Round getRound() {
    return round;
  }
  
  public void update() {
    if (round != null) {
      standingsComparator = new StandingsComparator(round);
      players = (Players) round.getTournament().getPlayers().clone();
      //Sort players according to result
      Collections.sort(players, standingsComparator);
    }
    else {
      standingsComparator = new StandingsComparator(null);
      players = null;
    }
    initColumnNames();
    fireTableDataChanged();
  }
  
  void initColumnNames() {
    columnNames.clear();
    columnNames.add("Plac");
    columnNames.add("Namn");
    columnNames.add("Grp");
    columnNames.add("Klubb");
    if (settings.getShowLASK()) {
      columnNames.add("Rating");
    }
    if (settings.getShowELO()) {
      columnNames.add("ELO");
    }
    columnNames.add("Pong");
    // Add tiebreak columns
    if (round != null) {
      Vector<Tiebreak> tiebreaks = round.getTournament().getSelectedTiebreaks();
      for (int i=0; i<tiebreaks.size(); i++) {
        columnNames.add(tiebreaks.get(i).getShortName());
      }
    }
    this.fireTableStructureChanged();
  }
  
}
