package cp.gui.web;


import cp.Game;
import cp.Player;
import cp.Players;
import cp.Result;
import cp.Round;
import cp.Rounds;
import cp.gui.StandingsComparator;
import cp.Tournament;

import cp.db.*;
import cp.tiebreak.Tiebreak;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.DecimalFormat;

import java.util.Collections;

import java.util.Vector;

import javax.swing.JOptionPane;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

public class HTMLRounds extends HTMLPairings {
  public void publish(Tournament tournament) throws FileNotFoundException, IOException {
    // Create output html file
    String fileName = tournament.getRoundForRoundPage();
    if (fileName != null) {
      try {
        // Create document
        Document document = createDocument();
        Element html = document.createElement("html");
        document.appendChild(html);
        // Create title
        Element head = document.createElement("head");
        html.appendChild(head);
        Element title = document.createElement("title");
        head.appendChild(title);
				// Create stylesheet
				appendStylesheet(document, head);
        Text text = document.createTextNode("Rond fr rond " + tournament);
        title.appendChild(text);
        // Create body
        Element body = document.createElement("body");
        html.appendChild(body);
        text = document.createTextNode("Lotta - Resultat fr turnering " + tournament.toString());
        body.appendChild(text);
        
        // Save the table
        save(document, body, tournament);
        
        // Save the file
        save(document, fileName);
      }
      catch (ParserConfigurationException e) {
        JOptionPane.showMessageDialog(null, "ParserConfigurationException:" + e.getMessage(), "Kan inte skapa fil", JOptionPane.ERROR_MESSAGE);
      }
      catch (TransformerConfigurationException e) {
        JOptionPane.showMessageDialog(null, "Resultatfil kan inte ppnas eller skapas. Meddelande:" + e.getMessage(), "Kan inte ppna fil", JOptionPane.ERROR_MESSAGE);
      }
      catch (TransformerException e) {
        JOptionPane.showMessageDialog(null, "Resultatfil kan inte ppnas eller skapas. Meddelande:" + e.getMessage(), "Kan inte ppna fil", JOptionPane.ERROR_MESSAGE);
      }
    }
    else {
      JOptionPane.showMessageDialog(null, "Du mste ange ett namn fr html-filen i turneringsdialogen fr att kunna publicera den.", "Ingen html-fil specificerad", JOptionPane.INFORMATION_MESSAGE);
    }
  }
  
  void save(Document document, Element body, Tournament tournament) {
    Settings settings = tournament.getSettings();
    // Insert table
    Element table = document.createElement("table");
    body.appendChild(table);
    table.setAttribute("class", "CP_Table");
    table.setAttribute("cellpadding", "0");
    table.setAttribute("cellspacing", "0");
    table.setAttribute("border", "0");
    // Create header
    Element tr = document.createElement("tr");
    table.appendChild(tr);
    tr.setAttribute("class", "CP_TableHeader");
    Element th = document.createElement("th");
    tr.appendChild(th);
    Text text = document.createTextNode("Plac");
    th.appendChild(text);
    th.setAttribute("class", "CP_Place");

    th = document.createElement("th");
    tr.appendChild(th);
    text = document.createTextNode("Namn");
    th.appendChild(text);
    th.setAttribute("class", "CP_Player");

    th = document.createElement("th");
    tr.appendChild(th);
    text = document.createTextNode("Grp");
    th.appendChild(text);
    th.setAttribute("class", "CP_Group");

    if (settings.getShowLASK()) {
      th = document.createElement("th");
      tr.appendChild(th);
      text = document.createTextNode("Rating");
      th.appendChild(text);
      th.setAttribute("class", "CP_Rating");
    }

    if (settings.getShowELO()) {
      th = document.createElement("th");
      tr.appendChild(th);
      text = document.createTextNode("ELO");
      th.appendChild(text);
      th.setAttribute("class", "CP_Rating");
    }

    for (int i=0; i<tournament.getNrOfRounds(); i++) {
      th = document.createElement("th");
      tr.appendChild(th);
      text = document.createTextNode(String.format("%d", i+1));
      th.appendChild(text);
      th.setAttribute("class", "CP_Round");
    }

    th = document.createElement("th");
    tr.appendChild(th);
    text = document.createTextNode("Pong");
    th.appendChild(text);
    th.setAttribute("class", "CP_Score");

    Vector<Tiebreak> tiebreaks = tournament.getSelectedTiebreaks();
    for (int i=0; i<tiebreaks.size(); i++) {
      th = document.createElement("th");
      tr.appendChild(th);
      text = document.createTextNode(tiebreaks.get(i).getShortName());
      th.appendChild(text);
      th.setAttribute("class", "CP_Tiebreak");
    }

    if (settings.getShowLASK()) {
      th = document.createElement("th");
      tr.appendChild(th);
      text = document.createTextNode("+/-");
      th.appendChild(text);
      th.setAttribute("class", "CP_Ratingdiff");
    }
    
    // Create all rowd
    savePlayers(document, table, tournament);
  }

  private void savePlayers(Document document, Element table, Tournament tournament) {
    Settings settings = tournament.getSettings();
		Players players = tournament.getPlayers();
    //Sort players according to result
		Rounds rounds = tournament.getRounds();
		StandingsComparator comparer = new StandingsComparator(rounds.get(rounds.size()-1));
    Collections.sort(players, comparer);
    // Create a row for each player
    DecimalFormat formatter = new DecimalFormat("0.###");
    for (int i=0; i<players.size(); i++) {
			Player player = players.get(i);
      Element tr = document.createElement("tr");
      table.appendChild(tr);
      tr.setAttribute("class", "CP_Row");

      // Place
      Element td = document.createElement("td");
      tr.appendChild(td);
      Text text = document.createTextNode(String.format("%d",i+1));
/*      if (i > 0) {
        Player previous = players.get(i-1);
        if (comparer.compare(player, previous) != 0) { 
          td.appendChild(text);
        }
      }
      else {*/
        td.appendChild(text);
//      }
      td.setAttribute("class", "CP_Place");
      // Player name
      td = document.createElement("td");
      tr.appendChild(td);
      text = document.createTextNode(player.toString());
      td.appendChild(text);
      td.setAttribute("class", "CP_Player");
      // Group
      td = document.createElement("td");
      tr.appendChild(td);
      if (player.getGroup() != null) {
        text = document.createTextNode(player.getGroup());
        td.appendChild(text);
      }
      td.setAttribute("class", "CP_Group");
      // Rating
      if (settings.getShowLASK()) {
        td = document.createElement("td");
        tr.appendChild(td);
        text = document.createTextNode(String.format("%d", player.getRatingN()));
        if (player.getRatingN() > 0) {
          td.appendChild(text);
        }
        td.setAttribute("class", "CP_Rating");
      }
      if (settings.getShowELO()) {
        td = document.createElement("td");
        tr.appendChild(td);
        text = document.createTextNode(String.format("%d", player.getRatingI()));
        if (player.getRatingI() > 0) {
          td.appendChild(text);
        }
        td.setAttribute("class", "CP_Rating");
      }
      
      // Insert all rounds
      insertResults(document, tr, player, players, tournament);
      // Insert score
      td = document.createElement("td");
      tr.appendChild(td);
      td.setAttribute("class", "CP_Score");
			Round round = rounds.lastElement();    
      if (round != null) {
        text = document.createTextNode(Result.toString(player.getScore(round)));
        td.appendChild(text);
        // Insert tiebreaks
        Vector<Tiebreak> tiebreaks = tournament.getSelectedTiebreaks();
        for (int j=0; j<tiebreaks.size(); j++) {
          td = document.createElement("td");
          tr.appendChild(td);
          text = document.createTextNode(formatter.format(player.getTiebreak(tiebreaks.get(j), round)));
          td.appendChild(text);
          td.setAttribute("class", "CP_Tiebreak");
        }
      }
      else {
        text = document.createTextNode("-");
        td.appendChild(text);
      }
      // Rating diff
      if (settings.getShowLASK()) {
        td = document.createElement("td");
        tr.appendChild(td);
        if (player.getRatingN() > 0) {
          text = document.createTextNode(formatter.format(player.getRatingNChange()));
        }
        else {
          text = document.createTextNode("");
        }
        td.appendChild(text);
        td.setAttribute("class", "CP_Rating");
      }
    }
  }

  private void insertResults(Document document, Element tr, Player player, Players players, Tournament tournament) {
		Rounds rounds = tournament.getRounds();
    Vector games = player.getGames();
    for (int i=0; i<tournament.getNrOfRounds(); i++) {
			Round round = null;
      if (i < rounds.size()) {
        round = rounds.get(i);
      }
      Element td = document.createElement("td");
      tr.appendChild(td);
      Element div = document.createElement("div");
      td.appendChild(div);
      // Insert opponent number
			Game game = getGame(games, i+1);
			Player opponent = null;
      Text text;
      if (game != null) {
        if (game.getWhitePlayer() == player) {
          opponent = game.getBlackPlayer();
          div.setAttribute("class", "CP_White");
        }
        else {
          opponent = game.getWhitePlayer();
          div.setAttribute("class", "CP_Black");
        }
        // Now we have the opponent. Which number does he have in the list?
        if (opponent != null) {
          text = document.createTextNode(String.format("%d", players.indexOf(opponent)+1));
        }
        else {
          text = document.createTextNode("");
        }
        div.appendChild(text);
        // Insert result
        text = document.createTextNode(getResultText(player, game));
        td.appendChild(text);
        td.appendChild(document.createElement("br"));
        td.setAttribute("class", "CP_Round");
      }
      else {
        text = document.createTextNode("");
        td.appendChild(text);
        td.appendChild(document.createElement("br"));
        td.setAttribute("class", "CP_Round");
      }
    }
  }
  
  private Game getGame(Vector games, int roundNr) {
    if (games != null) {
      for (int i=0; i<games.size(); i++) {
				Game game = (Game)games.get(i);
        if (game.getRoundNr() == roundNr) {
          return game;
        }
      }
    }
    return null;
  }

  private String getResultText(Player player, Game game) {
		Result result = game.getResult();
    if (player == game.getWhitePlayer()) {
      if (result.isPlayed()) {
        return Result.toString(result.getWhiteScore());
      }
      else if (result.getResultType() == Result.ResultType.WHITE_WIN_WO) {
        return "i";
      }
      else if (result.getResultType() == Result.ResultType.BLACK_WIN_WO) {
        return "o";
      }
      else if (result.getResultType() == Result.ResultType.DOUBLE_WO) {
        return "o";
      }
      else if (result.getResultType() == Result.ResultType.POSTPONED) {
        return "u";
      }
      else if (result.getResultType() == Result.ResultType.CANCELLED) {
        return "-";
      }
    }
    else {
      if (result.isPlayed()) {
        return Result.toString(result.getBlackScore());
      }
      else if (result.getResultType() == Result.ResultType.WHITE_WIN_WO) {
        return "o";
      }
      else if (result.getResultType() == Result.ResultType.BLACK_WIN_WO) {
        return "i";
      }
      else if (result.getResultType() == Result.ResultType.DOUBLE_WO) {
        return "o";
      }
      else if (result.getResultType() == Result.ResultType.POSTPONED) {
        return "u";
      }
      else if (result.getResultType() == Result.ResultType.CANCELLED) {
        return "-";
      }
    }
    return " ";    
  }
  

}
